
# Copyright (C) 2010-2020 RhodeCode GmbH
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License, version 3
# (only), as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This program is dual-licensed. If you wish to learn more about the
# RhodeCode Enterprise Edition, including its added features, Support services,
# and proprietary license terms, please see https://rhodecode.com/licenses/

"""
SQLAlchemy Metadata and Session object
"""

from sqlalchemy.orm import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy.orm import Session as SA_Session
from rhodecode.lib.caching_query import ORMCache


__all__ = [
    'Base', 'Session', 'SA_Session',
    'raw_query_executor',
    'bind_engine_to_session',
    'get_engine'
]

# scoped_session.  Apply our custom CachingQuery class to it,
# using a callable that will associate the dictionary
# of regions with the Query.
# to use cache use this in query
# .options(FromCache("sqlalchemy_cache_type", "cachekey"))
session_factory = sessionmaker(
    expire_on_commit=True,
    future=True
)

Session = scoped_session(session_factory)

# The declarative Base
Base = declarative_base()

# pass empty regions, so we can fetch it on-demand inside ORMCache
cache = ORMCache(regions={})
cache.listen_on_session(Session)


def raw_query_executor(engine=None):
    """

    :param engine:
    :return:
    """
    if not engine:
        engine = Session.bind
    session = SA_Session(engine)
    return session


def get_engine():
    return Session.bind


def bind_engine_to_session(engine):
    Session.remove()
    Session.configure(bind=engine)
